# Webpack loader to list the environment
## Introduction

It list out all the environment variables required for the project to run during webpack build. This way we can check if all environment is set in our `env` file as well if we using for browser, we can find if we passing only the requried environment to expose to the public.

## Install

```sh
npm install @common-stack/env-list-loader
```

Have the env-config.js file under `config` folder like below

```javascript
# packages-modules/account-api/server/src/config/env-config.ts (example)
# name of the file need to be `env-config.ts`
# add environment variables between `/* start */`  and `/* end */`

export const config = cleanEnv(
    process.env,
    /* start */
    {
        MAIL_SEND_DEFAULT_EMAIL: str(),
        TEAM_INVITATION_TOKEN_SECRET: str({ devDefault: 'xxxxxx' }),
        INVITATION_TOKEN_SECRET: str({ devDefault: 'xxxxx' }),
        NAMESPACE: str({ default: 'default' }),
    } /* end */,

)

```

If it is web, have the `public-config.ts` file under `config` folder like below

```javascript
# name `const publicEnv =` need to be exact.
const publicEnv = ['NODE_ENV', 'GRAPHQL_URL', 'FACEBOOK_APP_ID', 'GA_ID', 'LOG_LEVEL'];
```

In webpack config add as follows
```javascript
const EnvListPlugin = require('@common-stack/env-list-loader');

module.exports = {

  // ...
  module: {
    rules: [
            {
                // searches for files ends with <dir>/config/env-config.js or <dir>/config/public-config.js
                test: /config\/(env-config|public-config)\.(j|t)s/,
                use: {
                    loader: '@common-stack/env-list-loader',
                },
            },
      // ...
    ],
  },
    plugins: [
      // The plugin lists the environment that required as well recommendation about the keys used.
      new EnvListPlugin.Plugin(),
    ],
  }
}
```

In webpack log will see below

```
missed keys found
[
  'APP_URL',
  'CALLBACK_REDIRECT_URL',
  'POPUP_REDIRECT_URL',
  'DEFAULT_EXTENDED_RENEWAL_TIME',
  'WEB_APP_URL',
  'LOGOUT_REDIRECT_PATH',
  'LOGIN_REDIRECT_PATH'
]
unused keys found
[
  'NODE_ENV',
  'GRAPHQL_SUBSCRIPTION_URL',
  'FACEBOOK_APP_ID',
  'GA_ID',
  'ROUTER_HISTORY',
  'LOG_LEVEL',
  'CDE_WORKSPACE_DOMAIN',
  'CDE_WORKSPACE_URL_PRFIX',
  'ZIPKIN_URL',
  'EXTENSION_SOCKET_URL'
]
required env to set
[
  'CLIENT_URL',
  'AUTH0_CLIENT_ID',
  'AUTH0_DOMAIN',
  'AUTH0_API_AUDIENCE',
  'STRIPE_PUBLISHABLE_KEY'
]

```

Based on the above information, we can cross check if our environment file have the required environment variables.

In browser, we can update the `const publicEnv` value to include all `missed keys found` and remove `unused keys found` 

references: https://github.com/crystal-ball/svg-symbol-sprite-loader
